@@ -73,7 +73,7 @@ static gint                 gda_data_access_wrapper_get_n_rows      (GdaDataMode
 static gint                 gda_data_access_wrapper_get_n_columns   (GdaDataModel *model);
 static GdaColumn           *gda_data_access_wrapper_describe_column (GdaDataModel *model, gint col);
 static GdaDataModelAccessFlags gda_data_access_wrapper_get_access_flags(GdaDataModel *model);
-static const GValue      *gda_data_access_wrapper_get_value_at    (GdaDataModel *model, gint col, gint row);
+static const GValue        *gda_data_access_wrapper_get_value_at    (GdaDataModel *model, gint col, gint row, GError **error);
 static GdaValueAttribute    gda_data_access_wrapper_get_attributes_at (GdaDataModel *model, gint col, gint row);
 
 static void iter_row_changed_cb (GdaDataModelIter *iter, gint row, GdaDataAccessWrapper *model);
@@ -94,6 +94,7 @@ gda_data_access_wrapper_get_type (void)
 	static GType type = 0;
 
 	if (G_UNLIKELY (type == 0)) {
+		static GStaticMutex registering = G_STATIC_MUTEX_INIT;
 		static const GTypeInfo info = {
 			sizeof (GdaDataAccessWrapperClass),
 			(GBaseInitFunc) NULL,
@@ -112,8 +113,12 @@ gda_data_access_wrapper_get_type (void)
 			NULL
 		};
 
-		type = g_type_register_static (G_TYPE_OBJECT, "GdaDataAccessWrapper", &info, 0);
-		g_type_add_interface_static (type, GDA_TYPE_DATA_MODEL, &data_model_info);
+		g_static_mutex_lock (&registering);
+		if (type == 0) {
+			type = g_type_register_static (G_TYPE_OBJECT, "GdaDataAccessWrapper", &info, 0);
+			g_type_add_interface_static (type, GDA_TYPE_DATA_MODEL, &data_model_info);
+		}
+		g_static_mutex_unlock (&registering);
 	}
 	return type;
 }
@@ -129,7 +134,7 @@ gda_data_access_wrapper_class_init (GdaDataAccessWrapperClass *klass)
 	object_class->set_property = gda_data_access_wrapper_set_property;
         object_class->get_property = gda_data_access_wrapper_get_property;
 	g_object_class_install_property (object_class, PROP_MODEL,
-                                         g_param_spec_object ("model", "Data model being worked on", NULL,
+                                         g_param_spec_object ("model", NULL, "Data model being wrapped",
                                                               GDA_TYPE_DATA_MODEL,
 							      G_PARAM_READABLE | G_PARAM_WRITABLE |
 							      G_PARAM_CONSTRUCT_ONLY));
@@ -155,6 +160,7 @@ gda_data_access_wrapper_data_model_init (GdaDataModelClass *iface)
 	iface->i_iter_prev = NULL;
 
 	iface->i_set_value_at = NULL;
+	iface->i_iter_set_value = NULL;
 	iface->i_set_values = NULL;
         iface->i_append_values = NULL;
 	iface->i_append_row = NULL;
@@ -203,11 +209,6 @@ gda_data_access_wrapper_dispose (GObject *object)
 
 	/* free memory */
 	if (model->priv) {
-		if (model->priv->rows) {
-			g_hash_table_destroy (model->priv->rows);
-			model->priv->rows = NULL;
-		}
-
 		if (model->priv->iter) {
 			g_signal_handlers_disconnect_by_func (G_OBJECT (model->priv->iter),
 							      G_CALLBACK (iter_row_changed_cb), model);
@@ -277,20 +278,20 @@ gda_data_access_wrapper_set_property (GObject *object,
 				if (! (model->priv->model_access_flags & GDA_DATA_MODEL_ACCESS_RANDOM)) {
 					model->priv->iter = gda_data_model_create_iter (mod);
 					g_return_if_fail (model->priv->iter);
-					g_signal_connect (G_OBJECT (model->priv->iter), "row_changed",
+					g_signal_connect (G_OBJECT (model->priv->iter), "row-changed",
 							  G_CALLBACK (iter_row_changed_cb), model);
-					g_signal_connect (G_OBJECT (model->priv->iter), "end_of_data",
+					g_signal_connect (G_OBJECT (model->priv->iter), "end-of-data",
 							  G_CALLBACK (iter_end_of_data_cb), model);
 					model->priv->iter_row = -1; /* because model->priv->iter is invalid */
 					model->priv->rows = g_hash_table_new_full (g_direct_hash, g_direct_equal,
 										   NULL, (GDestroyNotify) g_object_unref);
 				}
 				else {
-					g_signal_connect (G_OBJECT (mod), "row_inserted",
+					g_signal_connect (G_OBJECT (mod), "row-inserted",
 							  G_CALLBACK (model_row_inserted_cb), model);
-					g_signal_connect (G_OBJECT (mod), "row_updated",
+					g_signal_connect (G_OBJECT (mod), "row-updated",
 							  G_CALLBACK (model_row_updated_cb), model);
-					g_signal_connect (G_OBJECT (mod), "row_removed",
+					g_signal_connect (G_OBJECT (mod), "row-removed",
 							  G_CALLBACK (model_row_removed_cb), model);
 				}
   
@@ -353,29 +354,6 @@ gda_data_access_wrapper_new (GdaDataModel *model)
 	return GDA_DATA_MODEL (retmodel);
 }
 
-/**
- * gda_data_access_wrapper_row_exists
- * @wrapper: a #GdaDataAccessWrapper objects
- * @row: a row number to test existance
- *
- * Tests if the wrapper model of @wrapper has a row number @row
- *
- * Returns: TRUE if row number @row exists
- */
-gboolean
-gda_data_access_wrapper_row_exists (GdaDataAccessWrapper *wrapper, gint row)
-{
-	g_return_val_if_fail (GDA_IS_DATA_ACCESS_WRAPPER (wrapper), FALSE);
-	
-	if (wrapper->priv->nb_cols == 0)
-		return FALSE;
-
-	if (gda_data_model_get_value_at ((GdaDataModel*) wrapper, 0, row))
-		return TRUE;
-	else
-		return FALSE;
-}
-
 /*
  * GdaDataModel interface implementation
  */
@@ -449,13 +427,22 @@ create_new_row (GdaDataAccessWrapper *model)
 	gint i;
 	GdaRow *row;
 
-	row = gda_row_new ((GdaDataModel *) model, model->priv->nb_cols);
+	row = gda_row_new (model->priv->nb_cols);
 	for (i = 0; i < model->priv->nb_cols; i++) {
 		GdaHolder *holder;
 
 		holder = gda_data_model_iter_get_holder_for_field (model->priv->iter, i);
-		if (holder)
-			gda_row_set_value (row, i, gda_holder_get_value (holder));
+		if (holder) {
+			GValue *dest;
+			const GValue *cvalue = gda_holder_get_value (holder);
+			dest = gda_row_get_value (row, i);
+			if (cvalue) {
+				gda_value_reset_with_type (dest, G_VALUE_TYPE ((GValue *) cvalue));
+				gda_value_set_from_value (dest, cvalue);
+			}
+			else
+				gda_value_set_null (dest);
+		}
 	}
 
 	g_hash_table_insert (model->priv->rows, GINT_TO_POINTER (model->priv->iter_row), row);
@@ -465,7 +452,7 @@ create_new_row (GdaDataAccessWrapper *model)
 }
 
 static const GValue *
-gda_data_access_wrapper_get_value_at (GdaDataModel *model, gint col, gint row)
+gda_data_access_wrapper_get_value_at (GdaDataModel *model, gint col, gint row, GError **error)
 {
 	GdaDataAccessWrapper *imodel;
 
@@ -476,13 +463,14 @@ gda_data_access_wrapper_get_value_at (GdaDataModel *model, gint col, gint row)
 	g_return_val_if_fail (row >= 0, NULL);
 
 	if (col >= imodel->priv->nb_cols) {
-		g_warning (_("Column %d out of range 0 - %d"), col, imodel->priv->nb_cols);
+		g_set_error (error, GDA_DATA_MODEL_ERROR, GDA_DATA_MODEL_COLUMN_OUT_OF_RANGE_ERROR,
+			     _("Column %d out of range (0-%d)"), col, imodel->priv->nb_cols - 1);
 		return NULL;
 	}
 
 	if (!imodel->priv->rows)
 		/* imodel->priv->model is a random access model, use it */
-		return gda_data_model_get_value_at (imodel->priv->model, col, row);
+		return gda_data_model_get_value_at (imodel->priv->model, col, row, error);
 	else {
 		GdaRow *gda_row;
 
@@ -497,8 +485,11 @@ gda_data_access_wrapper_get_value_at (GdaDataModel *model, gint col, gint row)
 					if (row == imodel->priv->iter_row)
 						return gda_row_get_value (gda_row, col);
 				}
-				else
+				else {
+					g_set_error (error, GDA_DATA_MODEL_ERROR, GDA_DATA_MODEL_ACCESS_ERROR,
+						      "%s", _("Can't set iterator's position"));
 					return NULL;
+				}
 			}
 				
 			gda_row = NULL;
@@ -516,13 +507,22 @@ gda_data_access_wrapper_get_value_at (GdaDataModel *model, gint col, gint row)
 				}
 			}
 
-			gda_row = g_hash_table_lookup (imodel->priv->rows, GINT_TO_POINTER (row));
+			if (! (imodel->priv->model_access_flags & GDA_DATA_MODEL_ACCESS_CURSOR_BACKWARD) ||
+			    ! (imodel->priv->model_access_flags & GDA_DATA_MODEL_ACCESS_CURSOR_FORWARD)) {
+				gda_row = g_hash_table_lookup (imodel->priv->rows, GINT_TO_POINTER (row));
 
-			if (gda_row)
+				if (gda_row)
+					return gda_row_get_value (gda_row, col);
+			}
+			else {
+				gda_row = create_new_row (imodel);
 				return gda_row_get_value (gda_row, col);
+			}
 		}
 	}
 
+	g_set_error (error, GDA_DATA_MODEL_ERROR, GDA_DATA_MODEL_ACCESS_ERROR,
+		      "%s", _("Can't access data"));
 	return NULL;
 }
 
@@ -568,7 +568,7 @@ gda_data_access_wrapper_get_attributes_at (GdaDataModel *model, gint col, gint r
 	imodel = (GdaDataAccessWrapper *) model;
 	g_return_val_if_fail (imodel->priv, 0);
 
-	if (!imodel->priv->rows)
+	if (imodel->priv->model)
 		flags = gda_data_model_get_attributes_at (imodel->priv->model, col, row);
 	flags |= GDA_VALUE_ATTR_NO_MODIF;
 	
